@@ -1,31 +1,38 @@
 `write.crn` <- function(crn, fname, header=NULL, append=FALSE)
 {
-    if(ncol(crn) != 2) stop("'crn' must have 2 columns")
+    if (ncol(crn) != 2) {
+        stop("'crn' must have 2 columns")
+    }
     crn2 <- crn
 
-    if(any(is.na(crn2))) {
+    if (any(is.na(crn2))) {
         na.flag <- is.na(crn2[, 1])
         crn2[na.flag, 2] <- 0
         crn2[na.flag, 1] <- 9.99
         print(head(crn2))
     }
 
-    if(append) {
-        if(!file.exists(fname))
+    if (append) {
+        if (!file.exists(fname)) {
             stop(gettextf("file %s does not exist, cannot append", fname))
-        if(length(header) > 0)
+        }
+        if (length(header) > 0) {
             stop("bad idea to append with 'header'")
+        }
     }
     header2 <- header
     if(length(header2) > 0){
-        if(!is.list(header2)) stop("header must be a list")
+        if (!is.list(header2)) {
+            stop("header must be a list")
+        }
         header.names <-
             c("site.id", "site.name", "spp.code", "state.country",
               "spp", "elev", "lat", "long", "first.yr", "last.yr",
               "lead.invs", "comp.date")
-        if(!all(header.names %in% names(header2)))
+        if (!all(header.names %in% names(header2))) {
             stop("'header' must be a list with the following names: ",
                  paste(dQuote(header.names), collapse = ", "))
+        }
         ## Record #1: 1-6 Site ID, 10-61 Site Name, 62-65 Species Code,
         ## optional ID#s
         ## Record #2: 1-6 Site ID, 10-22 State/Country, 23-40 Species,
@@ -44,7 +51,7 @@
         long <- header2$long[1]
         lead.invs <- header2$lead.invs[1]
         comp.date <- header2$comp.date[1]
-        lat.long <- ifelse(nchar(long) > 5, paste(lat, long, sep=""),
+        lat.long <- ifelse(nchar(long) > 5, paste0(lat, long),
                            paste(lat, long, sep=" "))
         yrs <- paste(header2$first.yr[1], header2$last.yr[1], sep=" ")
 
@@ -52,21 +59,22 @@
             c("site.id", "site.name", "spp.code", "state.country", "spp",
               "elev", "lat.long", "yrs", "lead.invs", "comp.date")
         field.width <- c(6, 52, 4, 13, 18, 5, 10, 9, 63, 8)
-        for(i in seq_along(field.name)){
+        for (i in seq_along(field.name)) {
             this.name <- field.name[i]
             this.width <- field.width[i]
             this.var <- get(this.name)
             this.nchar <- nchar(this.var)
-            if(this.nchar > this.width)
+            if (this.nchar > this.width) {
                 assign(this.name, substr(this.var, 1, this.width))
-            else if(this.nchar < this.width)
+            } else if (this.nchar < this.width) {
                 assign(this.name, encodeString(this.var, width = this.width))
+            }
         }
 
-        hdr1 <- paste(site.id, "   ", site.name, spp.code, sep="")
-        hdr2 <- paste(site.id, "   ", state.country, spp, elev, "  ",
-                      lat.long, "          ", yrs, sep="")
-        hdr3 <- paste(site.id, "   ", lead.invs, comp.date, sep="")
+        hdr1 <- paste0(site.id, "   ", site.name, spp.code)
+        hdr2 <- paste0(site.id, "   ", state.country, spp, elev, "  ",
+                       lat.long, "          ", yrs)
+        hdr3 <- paste0(site.id, "   ", lead.invs, comp.date)
         hdr <- c(hdr1, hdr2, hdr3)
     }
 
@@ -78,16 +86,17 @@
     crn.name <- names(crn2)[1]
     crn.width <- nchar(crn.name)
     ## If crn.width > 6, truncate
-    if(crn.width > 6)
+    if (crn.width > 6) {
         crn.name <- substr(crn.name, 1, 6)
-    else if(crn.width < 6) # Pad to nchar 6 (no leading zero)
+    } else if (crn.width < 6) {# Pad to nchar 6 (no leading zero)
         crn.name <- formatC(crn.name, width = 6, format = "f")
+    }
 
     dec.str <- character(n.decades)
-    for(i in seq_len(n.decades)){
+    for (i in seq_len(n.decades)) {
         ## 7-10 decade column
         dec <- decades[i]
-        dec.idx <- which(decades.vec%in%dec)
+        dec.idx <- which(decades.vec %in% dec)
         n.yrs <- length(dec.idx)
         dec.yrs <- yrs[dec.idx]
         ## Pad to nchar 4 (no leading zero)
@@ -103,18 +112,21 @@
         dec.samp.depth <- formatC(dec.samp.depth,
                                   digits = 0, width = 3, format = "f")
         ## Pad front end
-        if(i == 1) tmp <- paste(rep("9990  0", 10-n.yrs), collapse="")
-        else tmp <- ""
+        if (i == 1) {
+            tmp <- paste(rep("9990  0", 10-n.yrs), collapse="")
+        } else {
+            tmp <- ""
+        }
         ## Put it all together
         dec.str[i] <-
-            paste(crn.name, dec.yrs[1], tmp,
-                  paste(dec.rwi, dec.samp.depth, sep="", collapse=""),
-                  sep="")
+            paste0(crn.name, dec.yrs[1], tmp,
+                   paste0(dec.rwi, dec.samp.depth, collapse=""))
     }
     ## Finish last decade with 9990 as NA and 0 as samp depth.
-    dec.str[i] <- paste(dec.str[i],
-                        paste(rep("9990  0", 10-n.yrs), collapse=""),
-                        sep="")
-    if(length(header2) > 0) dec.str <- c(hdr, dec.str)
+    dec.str[i] <- paste0(dec.str[i],
+                         paste(rep("9990  0", 10-n.yrs), collapse=""))
+    if (length(header2) > 0) {
+        dec.str <- c(hdr, dec.str)
+    }
     cat(dec.str, file = fname, sep = "\n", append=append)
 }
